@@ -1,20 +1,45 @@
 #![cfg_attr(not(feature = "std"), no_std)]
 
+pub use bindings::*;
+pub use error::*;
+pub use mutex::EspMutex;
+
 pub mod error;
 pub mod mutex;
 
 mod alloc;
 mod panic;
+mod start;
+
+// (Temporary code) ESP-IDF does not (yet) have a pthread rwlock implementation, which is required by STD
+// We provide a quick and very hacky implementation here
+#[cfg(feature = "std")]
 mod pthread_rwlock;
 
-// ESP-IDF current stable version (4.3) has atomics for ESP32S2, but not for ESP32C3
+// (Temporary code) ESP-IDF current stable version (4.3) has atomics for ESP32S2, but not for ESP32C3
 // The ESP-IDF master branch has atomics for both
 #[cfg(all(esp32c3, esp_idf_version = "4.3"))]
 mod atomics_esp32c3;
 
-pub use bindings::*;
-pub use error::*;
-pub use mutex::EspMutex;
+/// A hack to make sure that the rwlock implementation and the esp32c3 atomics are linked to the final executable
+/// Call this function once e.g. in the beginning of your main function
+///
+/// This function will become no-op once ESP-IDF V4.4 is released
+pub fn link_patches() -> (*mut c_types::c_void, *mut c_types::c_void) {
+    #[cfg(feature = "std")]
+    let rwlock = pthread_rwlock::link_patches();
+
+    #[cfg(not(feature = "std"))]
+    let rwlock = core::ptr::null_mut();
+
+    #[cfg(all(esp32c3, esp_idf_version = "4.3"))]
+    let atomics = atomics_esp32c3::link_patches();
+
+    #[cfg(not(all(esp32c3, esp_idf_version = "4.3")))]
+    let atomics = core::ptr::null_mut();
+
+    (rwlock, atomics)
+}
 
 #[cfg(feature = "std")]
 #[allow(non_upper_case_globals)]
@@ -59,11 +84,5 @@ pub mod c_types {
 mod bindings {
     use super::c_types;
 
-    /// Do NOT use. This static mut is declared only as a workaround for the fact that libstd - in the link order -
-    /// is *following* esp-idf-sys, which means that unless we reference outrselves the pthread_rwlock_* symbols,
-    /// these will not be linked!
-    pub static mut __PTHREAD_RWLOCK_INTERNAL_REFERENCE: *mut c_types::c_void =
-        super::pthread_rwlock::pthread_rwlock_init as *mut _;
-
     include!(env!("EMBUILD_GENERATED_BINDINGS_FILE"));
 }
